import json
import logging
import csv
from time import sleep
from datetime import datetime
import paho.mqtt.client as mqtt

# 配置日志
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

class Mqtt:
    def __init__(self, broker_address, port):
        """
        初始化 MQTT 客户端

        :param broker_address: MQTT 代理地址
        :param port: MQTT 代理端口
        """
        self.broker_address = broker_address
        self.port = port
        self.client = None

    def connect(self):
        """
        连接到 MQTT 服务器
        """
        self.client = mqtt.Client()
        self.client.on_connect = self.on_connect  # 设置连接回调
        self.client.on_message = self.on_message  # 设置消息回调
        self.client.connect(self.broker_address, self.port)  # 连接到代理
        self.client.loop_start()  # 启动网络循环
        logging.info("已连接到MQTT 服务器")

    def disconnect(self):
        """
        断开与 MQTT 服务器的连接
        """
        if self.client:
            self.client.loop_stop()  # 停止网络循环
            self.client.disconnect()  # 断开连接
            logging.info("已断开与MQTT 服务器的连接")

    def on_connect(self, client, userdata, flags, rc):
        """
        连接成功或失败的回调函数

        :param client: 客户端实例
        :param userdata: 用户数据
        :param flags: 连接标志
        :param rc: 返回码
        """
        if rc == 0:
            logging.info("连接成功")
        else:
            logging.error(f"连接失败，返回码: {rc}")

    def on_message(self, client, userdata, msg):
        """
        接收到消息的回调函数

        :param client: 客户端实例
        :param userdata: 用户数据
        :param msg: 消息对象
        """
        logging.info(f"收到消息: {msg.topic} {msg.payload.decode()}")
        self.received_message = msg.payload.decode()  # 存储接收到的消息

    def subscribe(self, topic):
        """
        订阅指定的主题

        :param topic: 主题名称
        """
        if self.client:
            self.client.subscribe(topic)
            logging.info(f"已订阅主题: {topic}")

    def publish(self, topic, message):
        """
        发布消息到指定的主题

        :param topic: 主题名称
        :param message: 消息内容
        """
        if self.client:
            # 将消息转换为JSON字符串
            if isinstance(message, dict):
                message = json.dumps(message)
            self.client.publish(topic, message)
            logging.info(f"已发布消息到主题: {topic}")

    def wait_for_message(self, topic, timeout=5):
        """
        等待指定主题的消息

        :param topic: 主题名称
        :param timeout: 超时时间（秒）
        :return: 接收到的消息或 None
        """
        if self.client:
            start_time = datetime.now()
            while (datetime.now() - start_time).total_seconds() < timeout:
                if hasattr(self, 'received_message'):
                    return self.received_message
                sleep(0.1)
            return None
        return None

    @staticmethod
    def read_config_from_csv(file_path):
        """
        从 CSV 文件读取配置

        :param file_path: CSV 文件路径
        :return: 配置列表
        """
        with open(file_path, mode='r', encoding='utf-8') as file:
            reader = csv.DictReader(file)
            return list(reader)

    @staticmethod
    def build_message(config, current_time):
        """
        构建消息内容

        :param config: 配置字典
        :param current_time: 当前时间
        :return: 消息字典
        """
        return {
            "method": "/system/readSystemInfo",
            "clientId": config['clientId'],
            "result": json.dumps({
                "result": {
                    "buildInfo": {
                        "appToken": config['appToken'],
                        "companyNumber": config['companyNumber'],
                        "cnum": config['cnum'],
                        "conferenceName": "测试会议室",
                        "conferenceId": int(config['conferenceId']),
                        "defaultQrCodeUrl": "http://192.168.5.218:8888/group1/M00/00/21/wKgFyGNBWZmADtnNAAAwrxR0X8s016.png",
                        "aliasName": "zt",
                        "serverBaseUrl": "http://192.168.5.218:8996/",
                        "localBindTime": current_time,
                        "generalField": "{\"conferencePhone\":\"\",\"chooseTimeType\":1,\"meetingTopicSwitch\":\"1\",\"meetingContentSwitch\":\"1\",\"meetingReverseTypeSwitch\":\"1\",\"seatArrangeSwitch\":\"1\",\"meetingVoteSwitch\":\"1\",\"floorPlanPath\":\"\",\"jumpToPaperless\":2,\"approvalList\":[],\"isLeaderJoin\":false,\"meetingPublishScreenSwitch\":\"1\"}"
                    },
                    "faceVersion": "4.2.12021020201.1",
                    "wgtVersion": "0.0.81",
                    "deviceModel": "yx_rk3288",
                    "abiList": ["armeabi-v7a", "armeabi"],
                    "androidId": "48134e6047a19aaf",
                    "appName": "UBAINS",
                    "appPackageName": "com.ubains.local.gviewer",
                    "appVersion": 78,
                    "appVersionName": "1.1.78",
                    "battery": 0,
                    "bluetoothEnabled": False,
                    "camerasCount": 1,
                    "charging": True,
                    "density": 1,
                    "freeAndTotalMemory": "1176M/1997M",
                    "internalAvailableSize": 4306395136,
                    "internalTotalSize": 4877451264,
                    "ipAddress": "192.168.5.129",
                    "macAddress": config['macAddress'],
                    "networkType": "NETWORK_ETHERNET",
                    "processCpuUsage": "0.82%",
                    "resolution": "1280x800",
                    "romName": "rockchip",
                    "rooted": True,
                    "sdkVersionCode": 25,
                    "sdkVersionName": "7.1.2",
                    "sysDate": "Tue Oct 22 18:24:52 GMT+08:00 2024",
                    "sysDateStr": current_time,
                    "sysElapsedRealtime": "342:26:11",
                    "sysLanguage": "zh",
                    "sysSupportedSensors": ["Accelerometer sensor", "Gyroscope sensor", "Game Rotation Vector Sensor",
                                            "Gravity Sensor"],
                    "authCode": config['authCode'],
                    "conferenceName": "测试会议室"
                }
            })
        }

    def send_and_receive_messages(self, topic, message, num_times=1, timeout=5, interval=0.2):
        """
        发送并接收消息

        :param topic: 主题名称
        :param message: 消息内容
        :param num_times: 发送次数
        :param timeout: 超时时间（秒）
        :param interval: 每次发送之间的间隔时间（秒）
        """
        for i in range(num_times):
            try:
                self.publish(topic, message)
                received_message = self.wait_for_message(topic, timeout=timeout)
                logging.info(f"正在等待主题: {topic} 的消息，超时时间: {timeout} 秒")

                if received_message:
                    logging.info("消息接收成功！")
                else:
                    logging.warning("超时时间内未接收到消息。")

                sleep(interval)
            except Exception as e:
                logging.error(f"发送或接收消息时发生错误: {e}")
