import pytest
import logging
import os
import csv
from selenium.common import StaleElementReferenceException
from selenium.webdriver.common.by import By
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from 预定系统.base.Broswer_driver import Browser_driver

# 初始化日志记录器
logging.basicConfig(level=logging.ERROR, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

def read_csv_data(file_path=None):
    """
    读取 CSV 文件数据
    """
    # 初始化数据列表，用于存储 CSV 文件中的数据
    data = []

    # 使用环境变量或参数指定文件路径
    if file_path is None:
        file_path = os.getenv('CSV_FILE_PATH', '../测试数据/Login_Test.csv')

    try:
        # 打开 CSV 文件，使用 utf-8 编码读取
        with open(file_path, mode='r', encoding='utf-8') as csvfile:
            # 使用 csv.DictReader 读取 CSV 文件，自动将每行数据转换为字典
            csv_reader = csv.DictReader(csvfile)
            # 遍历每行数据，提取用户名、密码和验证码，将其作为元组添加到数据列表中
            for row in csv_reader:
                data.append((row['username'], row['password'], row['verification_code']))
    except FileNotFoundError:
        # 如果 CSV 文件未找到，记录错误日志并退出测试
        logger.error("CSV 文件未找到，请检查文件路径")
        pytest.exit("CSV 文件未找到，请检查文件路径")
    except csv.Error as e:
        # 如果 CSV 格式错误，记录错误日志并退出测试
        logger.error(f"CSV 格式错误: {e}")
        pytest.exit(f"CSV 格式错误: {e}")
    except Exception as e:
        # 如果有其他异常发生，记录错误日志并退出测试
        logger.error(f"出现错误: {e}")
        pytest.exit(f"出现错误: {e}")

    # 返回读取的数据列表
    return data


@pytest.fixture(scope='session')
def driver():
    """
    初始化浏览器并打开指定URL
    """
    url = 'https://nat.ubainsyun.com:11046'  # 通过配置文件获取URL
    # url = 'https://192.168.5.218'
    logging.info("'----------' 正在初始化浏览器 '----------'")
    try:
        driver = Browser_driver().browser_init()  # 初始化浏览器驱动
        logging.info("'----------' 浏览器初始化成功 '----------'")
    except Exception as e:
        logging.error(f"浏览器初始化失败: {e}")
        raise

    driver.implicitly_wait(10)  # 设置隐式等待时间为10秒
    driver.maximize_window()  # 最大化浏览器窗口
    logging.info("'----------' 正在打开URL '----------'")

    try:
        driver.get(url)
        logging.info(f"成功打开URL: {url}")
    except Exception as e:
        logging.error(f"打开URL失败: {e}")
        raise

    handle_ssl_warning(driver)  # 处理可能存在的SSL证书警告

    yield driver
    driver.quit()  # 在测试结束后关闭浏览器


def handle_ssl_warning(driver):
    """
    处理 SSL 证书警告
    此函数用于在网页浏览器中处理 SSL 证书错误。它通过点击特定的按钮来忽略 SSL 警告，以便在自动化测试中继续进行。

    参数:
    - driver: WebDriver实例，用于控制网页浏览器的行为

    异常:
    - 如果在处理 SSL 警告过程中发生任何错误，会记录错误信息并抛出异常
    """
    try:
        # 记录处理 SSL 证书警告的开始信息
        logging.info("'----------' 正在处理SSL证书警告 '----------'")

        # 等待并点击“详细信息”按钮
        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="details-button"]'))
        ).click()

        # 等待并点击“继续”链接，以忽略 SSL 警告
        WebDriverWait(driver, 10).until(
            EC.element_to_be_clickable((By.XPATH, '//*[@id="proceed-link"]'))
        ).click()

        # 等待页面加载完成
        WebDriverWait(driver, 10).until(
            EC.presence_of_element_located((By.TAG_NAME, 'body'))
        )

        # 记录处理 SSL 证书警告完成的信息
        logging.info("'----------' SSL证书警告处理完成 '----------'")
    except Exception as e:
        # 记录处理 SSL 证书警告过程中遇到的错误，并抛出异常
        logging.error(f"处理 SSL 证书警告遇到错误: {e}")
        raise

def _find_element(driver, by, value):
    """
    查找元素，并处理可能的异常

    该函数尝试在给定的驱动程序(driver)中查找一个元素，该元素由定位方式(by)和定位值(value)共同确定。
    如果在查找过程中出现StaleElementReferenceException异常，即元素可能不再附属于当前页面，
    则函数会再次尝试查找该元素，以确保元素被正确找到并返回。

    参数:
    - driver: WebDriver实例，用于操作浏览器。
    - by: 字符串，指定查找元素的定位方式（如ID, XPATH等）。
    - value: 字符串，定位方式对应的值。

    返回:
    - 找到的元素对象。
    """
    try:
        # 使用WebDriverWait等待最多20秒，直到元素被定位到。
        # 这里使用presence_of_element_located预期条件，以等待元素出现在DOM中。
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )
    except StaleElementReferenceException:
        # 如果抛出StaleElementReferenceException异常，说明页面可能已经变化，
        # 导致原先的元素引用失效。这时再次尝试查找元素。
        return WebDriverWait(driver, 20).until(
            EC.presence_of_element_located((by, value))
        )

def perform_login(driver, username, password, verification_code):
    """
    执行登录操作

    参数:
    - driver: 浏览器驱动
    - username: 用户名
    - password: 密码
    - verification_code: 验证码

    该函数通过浏览器驱动模拟用户登录过程，依次输入用户名、密码和验证码，然后点击登录按钮。
    登录后，根据返回的页面判断登录是否成功，并记录相应的日志信息。
    """
    # 隐式等待确保元素能被找到
    driver.implicitly_wait(5)
    # 刷新页面以确保登录页面最新
    driver.refresh()

    try:
        # 清除并输入用户名
        username_input = _find_element(driver, By.XPATH,
                                       '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[1]/div/input')
        username_input.clear()
        username_input.send_keys(username)

        # 清除并输入密码
        password_input = _find_element(driver, By.XPATH,
                                       '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[2]/div/input')
        password_input.clear()
        password_input.send_keys(password)

        # 清除并输入验证码
        verification_input = _find_element(driver, By.XPATH,
                                           '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[3]/div[1]/div/input')
        verification_input.clear()
        verification_input.send_keys(verification_code)

        # 点击登录按钮
        login_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[2]/div/form/div[4]/input')
        login_button.click()

        # 检查登录是否成功
        if check_login_null(driver):
            # 登录失败，必填项不能为空
            logger.info("登录失败，必填项不能为空！断言成功！")
        elif check_login_success(driver):
            # 登录成功
            logger.info("登录成功，断言成功！")
            return_to_login_page(driver)  # 返回登录页面
        elif check_login_failure(driver):
            # 登录失败，必填项错误
            logger.info("登录失败, 必填项错误！断言成功！")
        else:
            # 登录状态未知
            logger.error("登录状态未知，断言失败！")
            pytest.fail("登录状态未知，断言失败！")
    except StaleElementReferenceException as e:
        # 元素已失效，重新查找
        logger.error(f"元素已失效，重新查找... {e}")
        pytest.fail(f"元素已失效，重新查找... {e}")
    except Exception as e:
        # 登录操作失败
        logger.error(f"登录操作失败: {e}")
        pytest.fail(f"登录操作失败: {e}")


def return_to_login_page(driver):
    """
    返回登录页面

    通过点击退出按钮操作，实现从当前页面返回登录页面的功能。

    参数:
    - driver: WebDriver实例，用于操作浏览器。

    返回值:
    无
    """
    try:
        # 定位到退出按钮并点击
        logout_button = _find_element(driver, By.XPATH, '//*[@id="app"]/div/div[1]/div/img[3]')
        logout_button.click()

        # 记录日志，表明已成功尝试返回登录页面
        logging.info("已成功返回登录页面")

        # 刷新页面，并等待一段时间以确保页面加载完成
        driver.refresh()
        driver.implicitly_wait(5)
    except Exception as e:
        # 如果发生异常，记录错误日志并使测试用例失败
        logger.error(f"返回登录页面失败: {e}")
        pytest.fail(f"返回登录页面失败: {e}")

def check_login_null(driver):
    """
    检查登录失败，必填项为空的情况
    """
    try:
        # 使用WebDriverWait等待器，等待最多10秒，直到登录失败的提示元素可见
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '/html/body/div[2]/p'))
        ).text

        # 断言登录失败后，提示信息中包含'请输入'关键词
        assert "请输入" in login_element_text, "登录失败后未找到'请输入'提示语"
        return True
    except Exception as e:
        # 异常情况下，记录错误日志，并返回False表示检查未通过
        logger.error(f"登录失败检查失败: {e}")
        return False

def check_login_success(driver):
    """
    检查登录成功的情况
    """
    try:
        # 等待并获取登录成功后的欢迎信息
        message_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, "//*[contains(text(), '欢迎')]"))
        ).text

        # 断言欢迎信息是否符合预期
        assert "欢迎 预定标准版测试" == message_element_text, "登录成功后找到'欢迎 预定标准版测试'提示语"

        # 如果一切正常，返回True
        return True
    except Exception as e:
        # 如果检查过程中发生异常，记录错误信息，并返回False
        logger.error(f"登录成功检查失败: {e}")
        return False

def check_login_failure(driver):
    """
    检查登录失败的情况
    """
    try:
        # 使用WebDriverWait等待，直到定位到特定XPath的元素可见，获取该元素的文本
        login_element_text = WebDriverWait(driver, 10).until(
            EC.visibility_of_element_located((By.XPATH, '//*[@id="app"]/div/div[3]/div[1]/div[1]/div[1]'))
        ).text

        # 断言登录元素的文本是否为"账号登录"，如果符合，则说明登录失败检查成功
        assert "账号登录" == login_element_text, "登录失败后未找到'账号登录'提示语"
        return True
    except Exception as e:
        # 如果发生异常，记录错误信息并返回False，表示登录失败检查失败
        logger.error(f"登录失败检查失败: {e}")
        return False


@pytest.mark.parametrize("username,password,verification_code", read_csv_data())
def test_login_from_csv(driver, username, password, verification_code):
    """
    从 CSV 文件读取并测试登录情况
    """
    # 初始化日志记录，标记测试开始
    logger.info(f"'----------' 开始执行测试 '----------'")

    # 执行登录操作，传递驱动、用户名、密码和验证码
    perform_login(driver, username, password, verification_code)

    # 登录操作完成后，记录日志，标记测试结束
    logger.info(f"'----------' 执行完成测试 '----------'")

    # 隐式等待，确保后续操作前页面加载完成
    driver.implicitly_wait(10)


if __name__ == '__main__':
    pytest.main()

# import pytest
# import logging
# import os
# import csv
# from selenium.webdriver.common.by import By
# from selenium.webdriver.support.ui import WebDriverWait
# from selenium.webdriver.support import expected_conditions as EC
# from selenium.common import StaleElementReferenceException
# from selenium.webdriver import Chrome
#
# from 登录模块.base.Base import Base
# from 登录模块.base.Broswer_driver import Browser_driver
#
# # 初始化日志记录器
# logging.basicConfig(level=logging.ERROR, format='%(asctime)s - %(levelname)s - %(message)s')
# logger = logging.getLogger(__name__)
#
# def read_csv_data(file_path=None):
#     """
#     读取 CSV 文件数据
#     """
#     data = []
#     if file_path is None:
#         file_path = os.getenv('CSV_FILE_PATH', 'Login_Test.csv')
#
#     try:
#         with open(file_path, mode='r', encoding='utf-8') as csvfile:
#             csv_reader = csv.DictReader(csvfile)
#             for row in csv_reader:
#                 data.append((row['username'], row['password'], row['verification_code']))
#     except FileNotFoundError:
#         logger.error("CSV 文件未找到，请检查文件路径")
#         pytest.exit("CSV 文件未找到，请检查文件路径")
#     except csv.Error as e:
#         logger.error(f"CSV 格式错误: {e}")
#         pytest.exit(f"CSV 格式错误: {e}")
#     except Exception as e:
#         logger.error(f"出现错误: {e}")
#         pytest.exit(f"出现错误: {e}")
#
#     return data
#
# @pytest.fixture(scope='session')
# def driver():
#     """
#     初始化浏览器并打开指定URL
#     """
#     url = 'https://nat.ubainsyun.com:11046'
#     logging.info("'----------' 正在初始化浏览器 '----------'")
#     try:
#         driver = Browser_driver().browser_init()
#         logging.info("'----------' 浏览器初始化成功 '----------'")
#     except Exception as e:
#         logging.error(f"浏览器初始化失败: {e}")
#         raise
#
#     driver.implicitly_wait(10)
#     driver.maximize_window()
#     logging.info("'----------' 正在打开URL '----------'")
#
#     try:
#         driver.get(url)
#         logging.info(f"成功打开URL: {url}")
#     except Exception as e:
#         logging.error(f"打开URL失败: {e}")
#         raise
#
#     yield driver
#     driver.quit()
#
# def test_login_from_csv(driver):
#     """
#     从 CSV 文件读取并测试登录情况
#     """
#     logger.info(f"'----------' 开始执行测试 '----------'")
#     base = Base(driver)
#
#     for username, password, verification_code in read_csv_data():
#         base.perform_login(username, password, verification_code)
#
#     logger.info(f"'----------' 执行完成测试 '----------'")
#     driver.implicitly_wait(10)
#
# if __name__ == '__main__':
#     pytest.main()
